# Python3.6
import matplotlib.pyplot as plt
import pandas as pd

import os


cols_labels_ex01 = ['Fitness',
                    'Reference temperature, component 1',
                    'Reference rate, component 1',
                    'Reference temperature, component 2',
                    'Reference rate, component 2']

img_dir = 'images'

tga_exp_labels = ['Temp', 'MLR']

"""
Create plots for example 1, only a simple tga simulation based on FDS example
'tga_analysis.fds'. This file has been run to create mock-up experimental data,
which is used to compare the performance of the IMP against. 
"""

# Provide file name.
sim_data_db_file_ex01 = 'example1/tga_analysis_01/propti_db.csv'

sim_data_db_file_ex02 = 'example2/tga_analysis_02/propti_db.csv'
sim_data_5k_file_ex02 = 'example2/tga_analysis_02/tga_5k/tga_5k_tga.csv'
sim_data_10k_file_ex02 = 'example2/tga_analysis_02/tga_10k/tga_10k_tga.csv'
sim_data_15k_file_ex02 = 'example2/tga_analysis_02/tga_15k/tga_15k_tga.csv'


####
# Read simulation data.
# Example 1
sim_data_db_ex01 = pd.read_csv(sim_data_db_file_ex01, header=0)

# Example 2
sim_data_db_ex02 = pd.read_csv(sim_data_db_file_ex02, header=0)
sim_data_5k_ex02 = pd.read_csv(sim_data_5k_file_ex02, header=1)
sim_data_10k_ex02 = pd.read_csv(sim_data_10k_file_ex02, header=1)
sim_data_15k_ex02 = pd.read_csv(sim_data_15k_file_ex02, header=1)


####
# Read mock-up experimental data.
# Example 1
exp_file_ex01 = 'example1/tga_analysis_01/OriginalFiles/tga_10K_tga.csv'
exp_data_db_ex01 = pd.read_csv(exp_file_ex01,
                               header=1, usecols=tga_exp_labels)
# Example 2
exp_file5k_ex02 = 'example2/tga_analysis_02/OriginalFiles/tga_5K_tga.csv'
exp_data5k_db_ex02 = pd.read_csv(exp_file5k_ex02,
                               header=1, usecols=tga_exp_labels)
exp_file10k_ex02 = 'example2/tga_analysis_02/OriginalFiles/tga_10K_tga.csv'
exp_data10k_db_ex02 = pd.read_csv(exp_file10k_ex02,
                               header=1, usecols=tga_exp_labels)
exp_file15k_ex02 = 'example2/tga_analysis_02/OriginalFiles/tga_15K_tga.csv'
exp_data15k_db_ex02 = pd.read_csv(exp_file15k_ex02,
                               header=1, usecols=tga_exp_labels)



# Collect column labels
cols_ex01 = list(sim_data_db_ex01)
cols_ex02 = list(sim_data_db_ex02)
print(cols_ex01)

####
# Fitness value development - scatter plot
plt.plot(sim_data_db_ex01[cols_ex01[0]],
         color='black', marker='.', linestyle='None')

# plt.title('Fitness development')
plt.xlabel('Individuals')
plt.ylabel('Fitness value')

plt.grid()
plt.tight_layout()

plt.savefig('{}/ex01_fitness_scatter.pdf'.format(img_dir), dpi=400)
plt.close()

print('Fitness scatter plot for example 1 created.')


####
# Ref. Temp. 1 development - scatter plot
plt.plot(sim_data_db_ex01[cols_ex01[1]],
         color='black', marker='.', linestyle='None')

# plt.title('Fitness development')
plt.xlabel('Individuals')
plt.ylabel('Ref. temp. comp. 1, in deg. Celsius ')

plt.grid()
plt.tight_layout()

plt.savefig('{}/ex01_reftemp01_scatter.pdf'.format(img_dir), dpi=400)
plt.close()

print('Ref. temp. comp. 1 scatter plot for example 1 created.')

####
# Ref. rate. 2 development - scatter plot
plt.plot(sim_data_db_ex01[cols_ex01[4]],
         color='black', marker='.', linestyle='None')

# plt.title('Fitness development')
plt.xlabel('Individuals')
plt.ylabel('Ref. rate comp. 2, in deg. Celsius ')

plt.grid()
plt.tight_layout()

plt.savefig('{}/ex01_refrate02_scatter.pdf'.format(img_dir), dpi=400)
plt.close()

print('Ref. rate comp. 2 scatter plot for example 1 created.')


####
# Read simulation results.
# Find best parameter set for example 1
best_index = sim_data_db_ex01['like1'].idxmax()
print(best_index)
print('Best parameters')
print('---------------')
rt1 = sim_data_db_ex01.iloc[best_index]['parrtc01']
print('Ref. temp. comp. 1: {}'.format(rt1))
rr1 = sim_data_db_ex01.iloc[best_index]['parrrc01']
print('Ref. rate comp. 1: {}'.format(rr1))
rt2 = sim_data_db_ex01.iloc[best_index]['parrtc02']
print('Ref. temp. comp. 2: {}'.format(rt2))
rr2 = sim_data_db_ex01.iloc[best_index]['parrrc02']
print('Ref. rate comp. 2: {}'.format(rr2))
print('---------------\n\n')

best_sim_values = []
for i in cols_ex01:
    if 'simulation' in i:
        new_value = sim_data_db_ex01.iloc[best_index][i]
        best_sim_values.append(new_value)

####
# Find best parameter set for example 2
best_index = sim_data_db_ex02['like1'].idxmax()
print(best_index)

best_sim_values_2 = []
for i in cols_ex01:
    if 'par' in i:
        new_value = sim_data_db_ex02.iloc[best_index][i]
        best_sim_values_2.append(new_value)


print(best_sim_values_2)

# sim_data_db_ex01.iloc[best_index]

#####
# Comparison between experiment and simulation
# Plot mock-up experiment
plt.plot(exp_data_db_ex01[tga_exp_labels[0]],
         exp_data_db_ex01[tga_exp_labels[1]],
         color='black', marker='.', linestyle='None',
         label='Mock-up experiment')

# Plot simulation
plt.plot(exp_data_db_ex01[tga_exp_labels[0]],
         best_sim_values,
         color='blue',
         label='Simulation')

# plt.title('Comparison mock-up experiment and SCEUA result')
plt.xlabel('Temperature, in deg. Celsius')
plt.ylabel('Mass loss rate, in g/s')
plt.xlim(200, 500)

plt.legend()

plt.grid()
plt.tight_layout()

plt.savefig('{}/ex01_exp_sim_comparison.pdf'.format(img_dir),
            dpi=400)
plt.close()

print('Comparison plot of simulation and experiment, for example 1 created.')


#####
# Plot generations as boxplot

def plot_box_rmse(df_name, plot_title, para_to_optimise,
                  num_complex, file_name=None, file_path=None):

    """
    Create a collection of box plots, one for each generation of the sceua.
    Aimed to better visualise the fitness value development over the inverse
    modelling process (IMP). It is tailored to the SCEUA from the Spotpy package
    :param df_name: name of the Pandas data frame
    :param plot_title: title of the plot
    :param para_to_optimise: number of parameters of the IMP
    :param num_complex: number of complexes of the IMP, using Spotpy SCEUA
    :param file_name: name of the PDF-file to be created
    :param file_path: path to the location where the file shall be written
    :return: creates a plot and saves it as PDF-file
    """

    # Message to indicate that the plotting process has started.
    n_plots = 1
    print("")
    print("RMSE box plot(s):")
    print("Start plotting, {} task(s) in queue.".format(n_plots))
    print("--------------")

    # Extract the total amount of individuals over all generations,
    # the very first individual will be skipped.
    individuals_total = len(df_name['chain'].tolist()) - 1
    # Debugging:
    #print 'Individuals total:', individuals_total

    # Calculate generation size
    generation_size = int((2 * para_to_optimise + 1) * num_complex)
    print(generation_size)

    # Calculate number of full generations. If last generation is
    # only partly complete it will be skipped.
    generations = individuals_total // generation_size
    print(generations)

    # Debugging:
    #print 'Generations:', generations

    # Check if at least one full generation exists, if not print
    # a message and terminate this particular task and go on with
    # the remaining tasks. Otherwise proceed creating generation
    # plots.
    if generations < 1:
        print("* Individuals do not fill a whole generation for task:")
        print("* {}".format(file_name))
        print("* Task of plotting generation data stopped.")
        return
    #continue

    # Prepare list of lists to take the data for multiple boxplots, based
    # on the generations.
    data_series_multi = []

    # Slice the column with the RMSE data according to generations.
    for i in range(generations):
        # Calculate where to start slicing (skip very first value).
        slice_begin = generation_size * i + 1
        # Calculate where to stop slicing (skip very first value).
        slice_end = generation_size * (i + 1) + 1
        # Slice data and convert into a list.
        new_data = df_name['like1'][slice_begin:slice_end].tolist()
        # Collect the lists for later plotting.
        data_series_multi.append(new_data)

    # Prepare plotting of multiple boxplots in one diagramm.
    multi_plot = plt.figure()
    # Call the subplots.
    ax = multi_plot.add_subplot(111)
    # Create multiple boxplots
    ax.boxplot(data_series_multi)

    # Finish the plot
    #pl.rcParams['figure.figsize'] = 16, 12
    # Prepare list for x-tick labels.
    x_tick_labels = ['',]
    # Create x-tick labels based on number of generations.
    for i in range(generations):
        x_tick_labels.append(str(i))
    # Set values for the x-ticks.
    plt.xticks(range(generations + 1),
               x_tick_labels, rotation='horizontal')
    plt.xlabel('Generations ({} individuals each)'.format(generation_size))
    plt.ylabel('Root Mean Square Error (RMSE)')
    # Create plot title from file name.
    # plt.title(plot_title)
    plt.grid()
    plt.tight_layout()

    plt.savefig('{}/{}_fitness_boxplot.pdf'.format(img_dir, file_name))
    plt.close(multi_plot)

    print("Plot '{}_boxplot.pdf' has been created.".format(file_name))
    # Message to indicate that the job is done.
    print("--------------")
    print("Queue of plotting tasks finished.")
    print("")


plot_box_rmse(sim_data_db_ex01, 'Fitness development, per generation', 4, 4, 'ex01')




#############
# Plotting example 2

####
# Fitness value development - scatter plot
plt.plot(sim_data_db_ex02[cols_ex02[0]],
         color='black', marker='.', linestyle='None')

# plt.title('Fitness development')
plt.xlabel('Individuals')
plt.ylabel('Fitness value')

plt.grid()
plt.tight_layout()

plt.savefig('{}/ex02_fitness_scatter.pdf'.format(img_dir), dpi=400)
plt.close()

print('Fitness scatter plot for example 2 created.')


#####
# Comparison between experiment and simulation
# Plot mock-up experiment
plt.plot(exp_data5k_db_ex02[tga_exp_labels[0]],
         exp_data5k_db_ex02[tga_exp_labels[1]],
         color='black', marker='.', linestyle='None',
         label='Mock-up experiment,   5 K/s')

plt.plot(exp_data10k_db_ex02[tga_exp_labels[0]],
         exp_data10k_db_ex02[tga_exp_labels[1]],
         color='blue', marker='.', linestyle='None',
         label='Mock-up experiment, 10 K/s')

plt.plot(exp_data15k_db_ex02[tga_exp_labels[0]],
         exp_data15k_db_ex02[tga_exp_labels[1]],
         color='green', marker='.', linestyle='None',
         label='Mock-up experiment, 15 K/s')


# Plot simulation
plt.plot(sim_data_5k_ex02[tga_exp_labels[0]],
         sim_data_5k_ex02[tga_exp_labels[1]],
         color='black',
         label='Simulation   5 K/s')
plt.plot(sim_data_10k_ex02[tga_exp_labels[0]],
         sim_data_10k_ex02[tga_exp_labels[1]],
         color='blue',
         label='Simulation 10 K/s')
plt.plot(sim_data_15k_ex02[tga_exp_labels[0]],
         sim_data_15k_ex02[tga_exp_labels[1]],
         color='green',
         label='Simulation 15 K/s')

# plt.title('Comparison mock-up experiment and SCEUA result')
plt.xlabel('Temperature, in deg. Celsius')
plt.ylabel('Mass loss rate, in g/s')

plt.xlim(200, 500)

plt.legend()

plt.grid()
plt.tight_layout()

plt.savefig('{}/ex02_exp_sim_comparison.pdf'.format(img_dir),
            dpi=400)
plt.close()

print('Comparison plot of simulation and experiment, for example 2 created.')


###
# Boxplot of generations
plot_box_rmse(sim_data_db_ex02, 'Fitness development, per generation', 4, 4, 'ex02')


####
# Arrhenius testing

arrh_f = 'arrhenius/TGA_analysis_02_5K_tga/rundir_wpm4o8fd (another copy)/TGA_analysis_02_tga.csv'
arrh_d = pd.read_csv(arrh_f, header=1)


plt.plot(exp_data5k_db_ex02[tga_exp_labels[0]],
         exp_data5k_db_ex02[tga_exp_labels[1]],
         color='black', marker='.', linestyle='None',
         label='Mock-up experiment, 5 K/s')


plt.plot(arrh_d[tga_exp_labels[0]],
         arrh_d[tga_exp_labels[1]],
         color='black',
         label='Arrhenius, 5 K/s')

plt.title('Comparison Arrhenius and ref. rate')
plt.xlabel('Temperature, in deg. Celsius')
plt.ylabel('Mass loss rate, in g/s')

plt.legend()

plt.grid()
plt.tight_layout()

plt.savefig('Arrhenius_sim_comparison.pdf',
            dpi=400)
plt.close()